# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .client import (
    ClientResource,
    AsyncClientResource,
    ClientResourceWithRawResponse,
    AsyncClientResourceWithRawResponse,
    ClientResourceWithStreamingResponse,
    AsyncClientResourceWithStreamingResponse,
)
from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .swarms import (
    SwarmsResource,
    AsyncSwarmsResource,
    SwarmsResourceWithRawResponse,
    AsyncSwarmsResourceWithRawResponse,
    SwarmsResourceWithStreamingResponse,
    AsyncSwarmsResourceWithStreamingResponse,
)

__all__ = [
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "SwarmsResource",
    "AsyncSwarmsResource",
    "SwarmsResourceWithRawResponse",
    "AsyncSwarmsResourceWithRawResponse",
    "SwarmsResourceWithStreamingResponse",
    "AsyncSwarmsResourceWithStreamingResponse",
    "ClientResource",
    "AsyncClientResource",
    "ClientResourceWithRawResponse",
    "AsyncClientResourceWithRawResponse",
    "ClientResourceWithStreamingResponse",
    "AsyncClientResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, TypedDict

from .agent_spec_param import AgentSpecParam

__all__ = ["SwarmSpecParam"]


class SwarmSpecParam(TypedDict, total=False):
    agents: Optional[Iterable[AgentSpecParam]]
    """
    A list of agents or specifications that define the agents participating in the
    swarm.
    """

    description: Optional[str]
    """
    A comprehensive description of the swarm's objectives, capabilities, and
    intended outcomes.
    """

    img: Optional[str]
    """
    An optional image URL that may be associated with the swarm's task or
    representation.
    """

    max_loops: Optional[int]
    """
    The maximum number of execution loops allowed for the swarm, enabling repeated
    processing if needed.
    """

    messages: Union[Iterable[Dict[str, object]], Dict[str, object], None]
    """A list of messages that the swarm should complete."""

    name: Optional[str]
    """
    The name of the swarm, which serves as an identifier for the group of agents and
    their collective task.
    """

    rearrange_flow: Optional[str]
    """Instructions on how to rearrange the flow of tasks among agents, if applicable."""

    return_history: Optional[bool]
    """
    A flag indicating whether the swarm should return its execution history along
    with the final output.
    """

    rules: Optional[str]
    """
    Guidelines or constraints that govern the behavior and interactions of the
    agents within the swarm.
    """

    service_tier: Optional[str]
    """The service tier to use for processing.

    Options: 'standard' (default) or 'flex' for lower cost but slower processing.
    """

    stream: Optional[bool]
    """A flag indicating whether the swarm should stream its output."""

    swarm_type: Optional[
        Literal[
            "AgentRearrange",
            "MixtureOfAgents",
            "SpreadSheetSwarm",
            "SequentialWorkflow",
            "ConcurrentWorkflow",
            "GroupChat",
            "MultiAgentRouter",
            "AutoSwarmBuilder",
            "HiearchicalSwarm",
            "auto",
            "MajorityVoting",
            "MALT",
            "DeepResearchSwarm",
            "CouncilAsAJudge",
            "InteractiveGroupChat",
        ]
    ]
    """
    The classification of the swarm, indicating its operational style and
    methodology.
    """

    task: Optional[str]
    """The specific task or objective that the swarm is designed to accomplish."""

    tasks: Optional[List[str]]
    """A list of tasks that the swarm should complete."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rate_get_limits_response import RateGetLimitsResponse as RateGetLimitsResponse



"""
Comprehensive unit tests for the Swarms API client.
Uses simple functions and loguru for logging.
"""

import asyncio
from typing import Any
from datetime import datetime

from loguru import logger

from swarms_client.client import (
    LogEntry,
    LogsResponse,
    SwarmsClient,
    AgentCompletionResponse,
    SwarmCompletionResponse,
)

# Test data
TEST_AGENT_CONFIG = {
    "agent_name": "Research Analyst",
    "description": "Expert at analyzing and synthesizing research data",
    "system_prompt": "You are a research analyst focused on providing detailed, accurate analysis.",
    "model_name": "gpt-4o",
    "temperature": 0.7,
    "max_tokens": 4096,
    "max_loops": 2,
}

TEST_AGENT_CONFIG_2 = {
    "agent_name": "Data Scientist",
    "description": "Expert at data analysis and visualization",
    "system_prompt": "You are a data scientist focused on extracting insights from data.",
    "model_name": "gpt-4o-mini",
    "temperature": 0.5,
    "max_tokens": 2048,
    "max_loops": 1,
}

TEST_SWARM_CONFIG = {
    "name": "Research Analysis Swarm",
    "description": "A swarm for comprehensive research analysis",
    "agents": [TEST_AGENT_CONFIG, TEST_AGENT_CONFIG_2],
    "max_loops": 3,
    "swarm_type": "SequentialWorkflow",
    "task": "Analyze the impact of artificial intelligence on healthcare in 2024",
    "return_history": True,
    "service_tier": "standard",
}

# Test results storage
test_results = {
    "total_tests": 0,
    "passed_tests": 0,
    "failed_tests": 0,
    "test_details": [],
}


def log_test_result(test_name: str, success: bool, error: str = None):
    """Log test result and update test statistics."""
    test_results["total_tests"] += 1
    if success:
        test_results["passed_tests"] += 1
        logger.info(f"✅ Test passed: {test_name}")
    else:
        test_results["failed_tests"] += 1
        logger.error(f"❌ Test failed: {test_name}")
        if error:
            logger.error(f"Error: {error}")

    test_results["test_details"].append(
        {
            "name": test_name,
            "success": success,
            "error": error,
            "timestamp": datetime.now().isoformat(),
        }
    )


def validate_response(response: Any, expected_type: type) -> bool:
    """Validate response type and structure."""
    try:
        if not isinstance(response, expected_type):
            return False
        return True
    except Exception:
        return False


def test_agent_completion():
    """Test single agent completion with a research task."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Example 1: Basic research task
        response = client.agent.create(
            agent_config=TEST_AGENT_CONFIG,
            task="Analyze the latest trends in quantum computing",
        )

        if not validate_response(response, AgentCompletionResponse):
            raise ValueError("Invalid response type")

        assert response.success
        assert response.name == TEST_AGENT_CONFIG["agent_name"]

        # Example 2: Task with history
        history = {
            "messages": [
                {
                    "role": "user",
                    "content": "What are the key challenges in quantum computing?",
                },
                {"role": "assistant", "content": "The main challenges include..."},
            ]
        }

        response_with_history = client.agent.create(
            agent_config=TEST_AGENT_CONFIG,
            task="Based on the previous discussion, what are the potential solutions?",
            # history=history,
        )

        print(response_with_history)
        assert response_with_history is not None

        log_test_result("test_agent_completion", True)
        return True
    except Exception as e:
        log_test_result("test_agent_completion", False, str(e))
        return False


def test_agent_batch_completion():
    """Test batch agent completions with multiple tasks."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Example: Multiple research tasks
        completions = [
            {
                "agent_config": TEST_AGENT_CONFIG,
                "task": "Analyze quantum computing trends",
            },
            {"agent_config": TEST_AGENT_CONFIG_2, "task": "Analyze AI in healthcare"},
            {
                "agent_config": {
                    "agent_name": "Market Analyst",
                    "description": "Expert at market analysis",
                    "model_name": "gpt-4o",
                    "temperature": 0.6,
                },
                "task": "Analyze the impact of AI on financial markets",
            },
        ]

        responses = client.agent.create_batch(completions)

        print(responses)

        # if not isinstance(responses, list):
        #     raise ValueError("Expected list of responses")

        assert responses is not None

        log_test_result("test_agent_batch_completion", True)
        return True
    except Exception as e:
        log_test_result("test_agent_batch_completion", False, str(e))
        return False


def test_swarm_completion():
    """Test single swarm completion with a complex research task."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Example 1: Sequential workflow
        response = client.swarm.create(**TEST_SWARM_CONFIG)

        if not validate_response(response, SwarmCompletionResponse):
            raise ValueError("Invalid response type")

        assert response.swarm_name == TEST_SWARM_CONFIG["name"]
        assert response.swarm_type == TEST_SWARM_CONFIG["swarm_type"]

        # Example 2: Group chat workflow
        group_chat_config = {
            "name": "AI Ethics Discussion",
            "description": "A group chat to discuss AI ethics",
            "agents": [
                {
                    "agent_name": "Ethicist",
                    "description": "Expert in AI ethics",
                    "model_name": "gpt-4o",
                    "temperature": 0.7,
                },
                {
                    "agent_name": "Technologist",
                    "description": "Expert in AI technology",
                    "model_name": "gpt-4o",
                    "temperature": 0.7,
                },
            ],
            "swarm_type": "GroupChat",
            "task": "Discuss the ethical implications of autonomous AI systems",
            "max_loops": 5,
        }

        group_chat_response = client.swarm.create(**group_chat_config)
        assert group_chat_response.success

        log_test_result("test_swarm_completion", True)
        return True
    except Exception as e:
        log_test_result("test_swarm_completion", False, str(e))
        return False


def test_swarm_batch_completion():
    """Test batch swarm completions with multiple workflows."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Example: Multiple research workflows
        swarms = [
            {
                **TEST_SWARM_CONFIG,
                "name": "Healthcare AI Research",
                "task": "Analyze AI applications in healthcare",
            },
            {
                **TEST_SWARM_CONFIG,
                "name": "Quantum Computing Research",
                "task": "Analyze quantum computing advances",
            },
            {
                "name": "Market Analysis Swarm",
                "description": "A swarm for market analysis",
                "agents": [
                    {
                        "agent_name": "Market Analyst",
                        "description": "Expert in market analysis",
                        "model_name": "gpt-4o",
                        "temperature": 0.6,
                    },
                    {
                        "agent_name": "Data Scientist",
                        "description": "Expert in data analysis",
                        "model_name": "gpt-4o-mini",
                        "temperature": 0.5,
                    },
                ],
                "swarm_type": "SequentialWorkflow",
                "task": "Analyze market trends in AI technology",
                "max_loops": 3,
            },
        ]

        responses = client.swarm.create_batch(swarms)

        if not isinstance(responses, list):
            raise ValueError("Expected list of responses")

        for response in responses:
            if not validate_response(response, SwarmCompletionResponse):
                raise ValueError("Invalid response type in batch")
            assert response.success

        log_test_result("test_swarm_batch_completion", True)
        return True
    except Exception as e:
        log_test_result("test_swarm_batch_completion", False, str(e))
        return False


def test_list_swarm_types():
    """Test listing available swarm types."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Test listing swarm types
        response = client.swarm.list_types()

        assert response.success
        assert isinstance(response.swarm_types, list)
        assert len(response.swarm_types) > 0

        # Verify common swarm types are present
        common_types = ["SequentialWorkflow", "GroupChat", "MixtureOfAgents"]
        for swarm_type in common_types:
            assert swarm_type in response.swarm_types

        log_test_result("test_list_swarm_types", True)
        return True
    except Exception as e:
        log_test_result("test_list_swarm_types", False, str(e))
        return False


def test_list_models():
    """Test listing available models."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Test listing models
        response = client.models.list()

        assert response.success
        assert isinstance(response.models, list)
        assert len(response.models) > 0

        # Verify common models are present
        common_models = ["gpt-4o", "gpt-4o-mini"]
        for model in common_models:
            assert model in response.models

        log_test_result("test_list_models", True)
        return True
    except Exception as e:
        log_test_result("test_list_models", False, str(e))
        return False


def test_list_logs():
    """Test listing API logs."""
    try:
        client = SwarmsClient(timeout=120, max_retries=3, retry_delay=2)

        # Test listing logs
        response = client.logs.list()

        # Convert response to proper format if needed
        if isinstance(response, dict):
            # Handle case where response is raw dict
            logs = response.get("logs", [])
            processed_logs = []
            for log in logs:
                # Convert id to string if it's an integer
                if isinstance(log.get("id"), int):
                    log["id"] = str(log["id"])

                # Convert data to dict if it's a list
                if isinstance(log.get("data"), list):
                    log["data"] = {"items": log["data"]}

                processed_logs.append(log)

            response = LogsResponse(
                status="success",
                count=len(processed_logs),
                logs=[LogEntry(**log) for log in processed_logs],
                timestamp=datetime.now().isoformat(),
            )

        assert isinstance(response, LogsResponse)
        assert isinstance(response.logs, list)

        # Verify log structure
        for log in response.logs:
            assert isinstance(log.id, str)
            assert isinstance(log.api_key, str)
            assert isinstance(log.data, dict)

        log_test_result("test_list_logs", True)
        return True
    except Exception as e:
        log_test_result("test_list_logs", False, str(e))
        return False


async def test_async_agent_completion():
    """Test async agent completion."""
    try:
        async with SwarmsClient(timeout=120, max_retries=3, retry_delay=2) as client:
            # Example: Async research task
            response = await client.agent.acreate(
                agent_config=TEST_AGENT_CONFIG,
                task="Analyze the latest trends in quantum computing",
            )

            if not validate_response(response, AgentCompletionResponse):
                raise ValueError("Invalid response type")

            assert response.success
            assert response.name == TEST_AGENT_CONFIG["agent_name"]

            log_test_result("test_async_agent_completion", True)
            return True
    except Exception as e:
        log_test_result("test_async_agent_completion", False, str(e))
        return False


async def test_async_swarm_completion():
    """Test async swarm completion."""
    try:
        async with SwarmsClient(timeout=120, max_retries=3, retry_delay=2) as client:
            # Example: Async research workflow
            response = await client.swarm.acreate(**TEST_SWARM_CONFIG)

            if not validate_response(response, SwarmCompletionResponse):
                raise ValueError("Invalid response type")

            assert response.swarm_name == TEST_SWARM_CONFIG["name"]
            assert response.swarm_type == TEST_SWARM_CONFIG["swarm_type"]

            log_test_result("test_async_swarm_completion", True)
            return True
    except Exception as e:
        log_test_result("test_async_swarm_completion", False, str(e))
        return False


def generate_markdown_report():
    """Generate a markdown report of test results."""
    success_rate = (
        (test_results["passed_tests"] / test_results["total_tests"] * 100)
        if test_results["total_tests"] > 0
        else 0
    )

    report = f"""# Swarms Client Test Report
Generated: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}

## Summary
- Total Tests: {test_results['total_tests']}
- Passed: {test_results['passed_tests']}
- Failed: {test_results['failed_tests']}
- Success Rate: {success_rate:.2f}%

## Test Details
"""

    for test in test_results["test_details"]:
        status = "✅ PASSED" if test["success"] else "❌ FAILED"
        report += f"""
### {test['name']}
- Status: {status}
- Timestamp: {test['timestamp']}
"""
        if not test["success"] and test["error"]:
            report += f"- Error: {test['error']}\n"

    return report


def run_tests():
    """Run all tests and generate report."""
    logger.info("Starting test suite...")

    # Run synchronous tests
    test_agent_completion()
    test_agent_batch_completion()
    test_swarm_completion()
    test_swarm_batch_completion()
    test_list_swarm_types()
    test_list_models()
    test_list_logs()

    # Run async tests
    asyncio.run(test_async_agent_completion())
    asyncio.run(test_async_swarm_completion())

    # Generate and save report
    report = generate_markdown_report()
    with open("test_report.md", "w") as f:
        f.write(report)

    logger.info("Test suite completed. Report generated: test_report.md")
    return test_results


if __name__ == "__main__":
    run_tests()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from swarms_client import SwarmsClient, AsyncSwarmsClient
from swarms_client.types import (
    SwarmRunResponse,
    SwarmGetLogsResponse,
    SwarmCheckAvailableResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSwarms:
    parametrize = pytest.mark.parametrize(
        "client", [False, True], indirect=True, ids=["loose", "strict"]
    )

    @pytest.mark.skip()
    @parametrize
    def test_method_check_available(self, client: SwarmsClient) -> None:
        swarm = client.swarms.check_available()
        assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_check_available(self, client: SwarmsClient) -> None:
        response = client.swarms.with_raw_response.check_available()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = response.parse()
        assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_check_available(self, client: SwarmsClient) -> None:
        with client.swarms.with_streaming_response.check_available() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = response.parse()
            assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_logs(self, client: SwarmsClient) -> None:
        swarm = client.swarms.get_logs()
        assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_logs(self, client: SwarmsClient) -> None:
        response = client.swarms.with_raw_response.get_logs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = response.parse()
        assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_logs(self, client: SwarmsClient) -> None:
        with client.swarms.with_streaming_response.get_logs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = response.parse()
            assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_run(self, client: SwarmsClient) -> None:
        swarm = client.swarms.run()
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_run_with_all_params(self, client: SwarmsClient) -> None:
        swarm = client.swarms.run(
            agents=[
                {
                    "agent_name": "agent_name",
                    "auto_generate_prompt": True,
                    "description": "description",
                    "max_loops": 0,
                    "max_tokens": 0,
                    "mcp_url": "mcp_url",
                    "model_name": "model_name",
                    "role": "role",
                    "streaming_on": True,
                    "system_prompt": "system_prompt",
                    "temperature": 0,
                    "tools_list_dictionary": [{"foo": "bar"}],
                }
            ],
            description="description",
            img="img",
            max_loops=0,
            messages=[{"foo": "bar"}],
            name="name",
            rearrange_flow="rearrange_flow",
            return_history=True,
            rules="rules",
            service_tier="service_tier",
            stream=True,
            swarm_type="AgentRearrange",
            task="task",
            tasks=["string"],
        )
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_run(self, client: SwarmsClient) -> None:
        response = client.swarms.with_raw_response.run()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = response.parse()
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_run(self, client: SwarmsClient) -> None:
        with client.swarms.with_streaming_response.run() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = response.parse()
            assert_matches_type(SwarmRunResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSwarms:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_check_available(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        swarm = await async_client.swarms.check_available()
        assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_check_available(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        response = await async_client.swarms.with_raw_response.check_available()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = await response.parse()
        assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_check_available(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        async with async_client.swarms.with_streaming_response.check_available() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = await response.parse()
            assert_matches_type(SwarmCheckAvailableResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_logs(self, async_client: AsyncSwarmsClient) -> None:
        swarm = await async_client.swarms.get_logs()
        assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_logs(self, async_client: AsyncSwarmsClient) -> None:
        response = await async_client.swarms.with_raw_response.get_logs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = await response.parse()
        assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_logs(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        async with async_client.swarms.with_streaming_response.get_logs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = await response.parse()
            assert_matches_type(SwarmGetLogsResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_run(self, async_client: AsyncSwarmsClient) -> None:
        swarm = await async_client.swarms.run()
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_run_with_all_params(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        swarm = await async_client.swarms.run(
            agents=[
                {
                    "agent_name": "agent_name",
                    "auto_generate_prompt": True,
                    "description": "description",
                    "max_loops": 0,
                    "max_tokens": 0,
                    "mcp_url": "mcp_url",
                    "model_name": "model_name",
                    "role": "role",
                    "streaming_on": True,
                    "system_prompt": "system_prompt",
                    "temperature": 0,
                    "tools_list_dictionary": [{"foo": "bar"}],
                }
            ],
            description="description",
            img="img",
            max_loops=0,
            messages=[{"foo": "bar"}],
            name="name",
            rearrange_flow="rearrange_flow",
            return_history=True,
            rules="rules",
            service_tier="service_tier",
            stream=True,
            swarm_type="AgentRearrange",
            task="task",
            tasks=["string"],
        )
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncSwarmsClient) -> None:
        response = await async_client.swarms.with_raw_response.run()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        swarm = await response.parse()
        assert_matches_type(SwarmRunResponse, swarm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_run(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        async with async_client.swarms.with_streaming_response.run() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            swarm = await response.parse()
            assert_matches_type(SwarmRunResponse, swarm, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.client.rate_get_limits_response import RateGetLimitsResponse

__all__ = ["RateResource", "AsyncRateResource"]


class RateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return RateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return RateResourceWithStreamingResponse(self)

    def get_limits(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateGetLimitsResponse:
        """
        Get the rate limits and current usage for the user associated with the provided
        API key.
        """
        return self._get(
            "/v1/rate/limits",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=RateGetLimitsResponse,
        )


class AsyncRateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncRateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return AsyncRateResourceWithStreamingResponse(self)

    async def get_limits(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RateGetLimitsResponse:
        """
        Get the rate limits and current usage for the user associated with the provided
        API key.
        """
        return await self._get(
            "/v1/rate/limits",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=RateGetLimitsResponse,
        )


class RateResourceWithRawResponse:
    def __init__(self, rate: RateResource) -> None:
        self._rate = rate

        self.get_limits = to_raw_response_wrapper(
            rate.get_limits,
        )


class AsyncRateResourceWithRawResponse:
    def __init__(self, rate: AsyncRateResource) -> None:
        self._rate = rate

        self.get_limits = async_to_raw_response_wrapper(
            rate.get_limits,
        )


class RateResourceWithStreamingResponse:
    def __init__(self, rate: RateResource) -> None:
        self._rate = rate

        self.get_limits = to_streamed_response_wrapper(
            rate.get_limits,
        )


class AsyncRateResourceWithStreamingResponse:
    def __init__(self, rate: AsyncRateResource) -> None:
        self._rate = rate

        self.get_limits = async_to_streamed_response_wrapper(
            rate.get_limits,
        )



from __future__ import annotations

import sys
import typing
import typing_extensions
from typing import Any, TypeVar, Iterable, cast
from collections import abc as _c_abc
from typing_extensions import (
    TypeIs,
    Required,
    Annotated,
    get_args,
    get_origin,
)

from ._utils import lru_cache
from .._types import InheritsGeneric
from .._compat import is_union as _is_union


def is_annotated_type(typ: type) -> bool:
    return get_origin(typ) == Annotated


def is_list_type(typ: type) -> bool:
    return (get_origin(typ) or typ) == list


def is_iterable_type(typ: type) -> bool:
    """If the given type is `typing.Iterable[T]`"""
    origin = get_origin(typ) or typ
    return origin == Iterable or origin == _c_abc.Iterable


def is_union_type(typ: type) -> bool:
    return _is_union(get_origin(typ))


def is_required_type(typ: type) -> bool:
    return get_origin(typ) == Required


def is_typevar(typ: type) -> bool:
    # type ignore is required because type checkers
    # think this expression will always return False
    return type(typ) == TypeVar  # type: ignore


_TYPE_ALIAS_TYPES: tuple[type[typing_extensions.TypeAliasType], ...] = (
    typing_extensions.TypeAliasType,
)
if sys.version_info >= (3, 12):
    _TYPE_ALIAS_TYPES = (*_TYPE_ALIAS_TYPES, typing.TypeAliasType)


def is_type_alias_type(tp: Any, /) -> TypeIs[typing_extensions.TypeAliasType]:
    """Return whether the provided argument is an instance of `TypeAliasType`.

    ```python
    type Int = int
    is_type_alias_type(Int)
    # > True
    Str = TypeAliasType("Str", str)
    is_type_alias_type(Str)
    # > True
    ```
    """
    return isinstance(tp, _TYPE_ALIAS_TYPES)


# Extracts T from Annotated[T, ...] or from Required[Annotated[T, ...]]
@lru_cache(maxsize=8096)
def strip_annotated_type(typ: type) -> type:
    if is_required_type(typ) or is_annotated_type(typ):
        return strip_annotated_type(cast(type, get_args(typ)[0]))

    return typ


def extract_type_arg(typ: type, index: int) -> type:
    args = get_args(typ)
    try:
        return cast(type, args[index])
    except IndexError as err:
        raise RuntimeError(
            f"Expected type {typ} to have a type argument at index {index} but it did not"
        ) from err


def extract_type_var_from_base(
    typ: type,
    *,
    generic_bases: tuple[type, ...],
    index: int,
    failure_message: str | None = None,
) -> type:
    """Given a type like `Foo[T]`, returns the generic type variable `T`.

    This also handles the case where a concrete subclass is given, e.g.
    ```py
    class MyResponse(Foo[bytes]):
        ...

    extract_type_var(MyResponse, bases=(Foo,), index=0) -> bytes
    ```

    And where a generic subclass is given:
    ```py
    _T = TypeVar('_T')
    class MyResponse(Foo[_T]):
        ...

    extract_type_var(MyResponse[bytes], bases=(Foo,), index=0) -> bytes
    ```
    """
    cls = cast(object, get_origin(typ) or typ)
    if cls in generic_bases:  # pyright: ignore[reportUnnecessaryContains]
        # we're given the class directly
        return extract_type_arg(typ, index)

    # if a subclass is given
    # ---
    # this is needed as __orig_bases__ is not present in the typeshed stubs
    # because it is intended to be for internal use only, however there does
    # not seem to be a way to resolve generic TypeVars for inherited subclasses
    # without using it.
    if isinstance(cls, InheritsGeneric):
        target_base_class: Any | None = None
        for base in cls.__orig_bases__:
            if base.__origin__ in generic_bases:
                target_base_class = base
                break

        if target_base_class is None:
            raise RuntimeError(
                "Could not find the generic base class;\n"
                "This should never happen;\n"
                f"Does {cls} inherit from one of {generic_bases} ?"
            )

        extracted = extract_type_arg(target_base_class, index)
        if is_typevar(extracted):
            # If the extracted type argument is itself a type variable
            # then that means the subclass itself is generic, so we have
            # to resolve the type argument from the class itself, not
            # the base class.
            #
            # Note: if there is more than 1 type argument, the subclass could
            # change the ordering of the type arguments, this is not currently
            # supported.
            return extract_type_arg(typ, index)

        return extracted

    raise RuntimeError(
        failure_message
        or f"Could not resolve inner type variable at index {index} for {typ}"
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .agent_run_params import AgentRunParams as AgentRunParams
from .agent_spec_param import AgentSpecParam as AgentSpecParam
from .swarm_run_params import SwarmRunParams as SwarmRunParams
from .swarm_spec_param import SwarmSpecParam as SwarmSpecParam
from .agent_run_response import AgentRunResponse as AgentRunResponse
from .swarm_run_response import SwarmRunResponse as SwarmRunResponse
from .agent_completion_param import AgentCompletionParam as AgentCompletionParam
from .swarm_get_logs_response import SwarmGetLogsResponse as SwarmGetLogsResponse
from .model_list_available_response import (
    ModelListAvailableResponse as ModelListAvailableResponse,
)
from .swarm_check_available_response import (
    SwarmCheckAvailableResponse as SwarmCheckAvailableResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..swarm_spec_param import SwarmSpecParam

__all__ = ["BatchRunParams"]


class BatchRunParams(TypedDict, total=False):
    body: Required[Iterable[SwarmSpecParam]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .batch_run_params import BatchRunParams as BatchRunParams
from .batch_run_response import BatchRunResponse as BatchRunResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import TypedDict

from .agent_spec_param import AgentSpecParam

__all__ = ["AgentRunParams"]


class AgentRunParams(TypedDict, total=False):
    agent_config: Optional[AgentSpecParam]
    """The configuration of the agent to be completed."""

    history: Union[Dict[str, object], Iterable[Dict[str, str]], None]
    """The history of the agent's previous tasks and responses.

    Can be either a dictionary or a list of message objects.
    """

    img: Optional[str]
    """
    An optional image URL that may be associated with the agent's task or
    representation.
    """

    imgs: Optional[List[str]]
    """
    A list of image URLs that may be associated with the agent's task or
    representation.
    """

    stream: Optional[bool]
    """A flag indicating whether the agent should stream its output."""

    task: Optional[str]
    """The task to be completed by the agent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from .swarms import (
    SwarmsResource,
    AsyncSwarmsResource,
    SwarmsResourceWithRawResponse,
    AsyncSwarmsResourceWithRawResponse,
    SwarmsResourceWithStreamingResponse,
    AsyncSwarmsResourceWithStreamingResponse,
)

__all__ = [
    "BatchResource",
    "AsyncBatchResource",
    "BatchResourceWithRawResponse",
    "AsyncBatchResourceWithRawResponse",
    "BatchResourceWithStreamingResponse",
    "AsyncBatchResourceWithStreamingResponse",
    "SwarmsResource",
    "AsyncSwarmsResource",
    "SwarmsResourceWithRawResponse",
    "AsyncSwarmsResourceWithRawResponse",
    "SwarmsResourceWithStreamingResponse",
    "AsyncSwarmsResourceWithStreamingResponse",
]



from __future__ import annotations

import io
import base64
import pathlib
from typing import Any, Mapping, TypeVar, cast
from datetime import date, datetime
from typing_extensions import (
    Literal,
    get_args,
    override,
    get_type_hints as _get_type_hints,
)

import anyio
import pydantic

from ._utils import (
    is_list,
    is_given,
    lru_cache,
    is_mapping,
    is_iterable,
)
from .._files import is_base64_file_input
from ._typing import (
    is_list_type,
    is_union_type,
    extract_type_arg,
    is_iterable_type,
    is_required_type,
    is_annotated_type,
    strip_annotated_type,
)
from .._compat import get_origin, model_dump, is_typeddict

_T = TypeVar("_T")


# TODO: support for drilling globals() and locals()
# TODO: ensure works correctly with forward references in all cases


PropertyFormat = Literal["iso8601", "base64", "custom"]


class PropertyInfo:
    """Metadata class to be used in Annotated types to provide information about a given type.

    For example:

    class MyParams(TypedDict):
        account_holder_name: Annotated[str, PropertyInfo(alias='accountHolderName')]

    This means that {'account_holder_name': 'Robert'} will be transformed to {'accountHolderName': 'Robert'} before being sent to the API.
    """

    alias: str | None
    format: PropertyFormat | None
    format_template: str | None
    discriminator: str | None

    def __init__(
        self,
        *,
        alias: str | None = None,
        format: PropertyFormat | None = None,
        format_template: str | None = None,
        discriminator: str | None = None,
    ) -> None:
        self.alias = alias
        self.format = format
        self.format_template = format_template
        self.discriminator = discriminator

    @override
    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(alias='{self.alias}', format={self.format}, format_template='{self.format_template}', discriminator='{self.discriminator}')"


def maybe_transform(
    data: object,
    expected_type: object,
) -> Any | None:
    """Wrapper over `transform()` that allows `None` to be passed.

    See `transform()` for more details.
    """
    if data is None:
        return None
    return transform(data, expected_type)


# Wrapper over _transform_recursive providing fake types
def transform(
    data: _T,
    expected_type: object,
) -> _T:
    """Transform dictionaries based off of type information from the given type, for example:

    ```py
    class Params(TypedDict, total=False):
        card_id: Required[Annotated[str, PropertyInfo(alias="cardID")]]


    transformed = transform({"card_id": "<my card ID>"}, Params)
    # {'cardID': '<my card ID>'}
    ```

    Any keys / data that does not have type information given will be included as is.

    It should be noted that the transformations that this function does are not represented in the type system.
    """
    transformed = _transform_recursive(data, annotation=cast(type, expected_type))
    return cast(_T, transformed)


@lru_cache(maxsize=8096)
def _get_annotated_type(type_: type) -> type | None:
    """If the given type is an `Annotated` type then it is returned, if not `None` is returned.

    This also unwraps the type when applicable, e.g. `Required[Annotated[T, ...]]`
    """
    if is_required_type(type_):
        # Unwrap `Required[Annotated[T, ...]]` to `Annotated[T, ...]`
        type_ = get_args(type_)[0]

    if is_annotated_type(type_):
        return type_

    return None


def _maybe_transform_key(key: str, type_: type) -> str:
    """Transform the given `data` based on the annotations provided in `type_`.

    Note: this function only looks at `Annotated` types that contain `PropertyInfo` metadata.
    """
    annotated_type = _get_annotated_type(type_)
    if annotated_type is None:
        # no `Annotated` definition for this type, no transformation needed
        return key

    # ignore the first argument as it is the actual type
    annotations = get_args(annotated_type)[1:]
    for annotation in annotations:
        if isinstance(annotation, PropertyInfo) and annotation.alias is not None:
            return annotation.alias

    return key


def _no_transform_needed(annotation: type) -> bool:
    return annotation == float or annotation == int


def _transform_recursive(
    data: object,
    *,
    annotation: type,
    inner_type: type | None = None,
) -> object:
    """Transform the given data against the expected type.

    Args:
        annotation: The direct type annotation given to the particular piece of data.
            This may or may not be wrapped in metadata types, e.g. `Required[T]`, `Annotated[T, ...]` etc

        inner_type: If applicable, this is the "inside" type. This is useful in certain cases where the outside type
            is a container type such as `List[T]`. In that case `inner_type` should be set to `T` so that each entry in
            the list can be transformed using the metadata from the container type.

            Defaults to the same value as the `annotation` argument.
    """
    if inner_type is None:
        inner_type = annotation

    stripped_type = strip_annotated_type(inner_type)
    origin = get_origin(stripped_type) or stripped_type
    if is_typeddict(stripped_type) and is_mapping(data):
        return _transform_typeddict(data, stripped_type)

    if origin == dict and is_mapping(data):
        items_type = get_args(stripped_type)[1]
        return {
            key: _transform_recursive(value, annotation=items_type)
            for key, value in data.items()
        }

    if (
        # List[T]
        (is_list_type(stripped_type) and is_list(data))
        # Iterable[T]
        or (
            is_iterable_type(stripped_type)
            and is_iterable(data)
            and not isinstance(data, str)
        )
    ):
        # dicts are technically iterable, but it is an iterable on the keys of the dict and is not usually
        # intended as an iterable, so we don't transform it.
        if isinstance(data, dict):
            return cast(object, data)

        inner_type = extract_type_arg(stripped_type, 0)
        if _no_transform_needed(inner_type):
            # for some types there is no need to transform anything, so we can get a small
            # perf boost from skipping that work.
            #
            # but we still need to convert to a list to ensure the data is json-serializable
            if is_list(data):
                return data
            return list(data)

        return [
            _transform_recursive(d, annotation=annotation, inner_type=inner_type)
            for d in data
        ]

    if is_union_type(stripped_type):
        # For union types we run the transformation against all subtypes to ensure that everything is transformed.
        #
        # TODO: there may be edge cases where the same normalized field name will transform to two different names
        # in different subtypes.
        for subtype in get_args(stripped_type):
            data = _transform_recursive(data, annotation=annotation, inner_type=subtype)
        return data

    if isinstance(data, pydantic.BaseModel):
        return model_dump(data, exclude_unset=True, mode="json")

    annotated_type = _get_annotated_type(annotation)
    if annotated_type is None:
        return data

    # ignore the first argument as it is the actual type
    annotations = get_args(annotated_type)[1:]
    for annotation in annotations:
        if isinstance(annotation, PropertyInfo) and annotation.format is not None:
            return _format_data(data, annotation.format, annotation.format_template)

    return data


def _format_data(
    data: object, format_: PropertyFormat, format_template: str | None
) -> object:
    if isinstance(data, (date, datetime)):
        if format_ == "iso8601":
            return data.isoformat()

        if format_ == "custom" and format_template is not None:
            return data.strftime(format_template)

    if format_ == "base64" and is_base64_file_input(data):
        binary: str | bytes | None = None

        if isinstance(data, pathlib.Path):
            binary = data.read_bytes()
        elif isinstance(data, io.IOBase):
            binary = data.read()

            if isinstance(binary, str):  # type: ignore[unreachable]
                binary = binary.encode()

        if not isinstance(binary, bytes):
            raise RuntimeError(
                f"Could not read bytes from {data}; Received {type(binary)}"
            )

        return base64.b64encode(binary).decode("ascii")

    return data


def _transform_typeddict(
    data: Mapping[str, object],
    expected_type: type,
) -> Mapping[str, object]:
    result: dict[str, object] = {}
    annotations = get_type_hints(expected_type, include_extras=True)
    for key, value in data.items():
        if not is_given(value):
            # we don't need to include `NotGiven` values here as they'll
            # be stripped out before the request is sent anyway
            continue

        type_ = annotations.get(key)
        if type_ is None:
            # we do not have a type annotation for this field, leave it as is
            result[key] = value
        else:
            result[_maybe_transform_key(key, type_)] = _transform_recursive(
                value, annotation=type_
            )
    return result


async def async_maybe_transform(
    data: object,
    expected_type: object,
) -> Any | None:
    """Wrapper over `async_transform()` that allows `None` to be passed.

    See `async_transform()` for more details.
    """
    if data is None:
        return None
    return await async_transform(data, expected_type)


async def async_transform(
    data: _T,
    expected_type: object,
) -> _T:
    """Transform dictionaries based off of type information from the given type, for example:

    ```py
    class Params(TypedDict, total=False):
        card_id: Required[Annotated[str, PropertyInfo(alias="cardID")]]


    transformed = transform({"card_id": "<my card ID>"}, Params)
    # {'cardID': '<my card ID>'}
    ```

    Any keys / data that does not have type information given will be included as is.

    It should be noted that the transformations that this function does are not represented in the type system.
    """
    transformed = await _async_transform_recursive(
        data, annotation=cast(type, expected_type)
    )
    return cast(_T, transformed)


async def _async_transform_recursive(
    data: object,
    *,
    annotation: type,
    inner_type: type | None = None,
) -> object:
    """Transform the given data against the expected type.

    Args:
        annotation: The direct type annotation given to the particular piece of data.
            This may or may not be wrapped in metadata types, e.g. `Required[T]`, `Annotated[T, ...]` etc

        inner_type: If applicable, this is the "inside" type. This is useful in certain cases where the outside type
            is a container type such as `List[T]`. In that case `inner_type` should be set to `T` so that each entry in
            the list can be transformed using the metadata from the container type.

            Defaults to the same value as the `annotation` argument.
    """
    if inner_type is None:
        inner_type = annotation

    stripped_type = strip_annotated_type(inner_type)
    origin = get_origin(stripped_type) or stripped_type
    if is_typeddict(stripped_type) and is_mapping(data):
        return await _async_transform_typeddict(data, stripped_type)

    if origin == dict and is_mapping(data):
        items_type = get_args(stripped_type)[1]
        return {
            key: _transform_recursive(value, annotation=items_type)
            for key, value in data.items()
        }

    if (
        # List[T]
        (is_list_type(stripped_type) and is_list(data))
        # Iterable[T]
        or (
            is_iterable_type(stripped_type)
            and is_iterable(data)
            and not isinstance(data, str)
        )
    ):
        # dicts are technically iterable, but it is an iterable on the keys of the dict and is not usually
        # intended as an iterable, so we don't transform it.
        if isinstance(data, dict):
            return cast(object, data)

        inner_type = extract_type_arg(stripped_type, 0)
        if _no_transform_needed(inner_type):
            # for some types there is no need to transform anything, so we can get a small
            # perf boost from skipping that work.
            #
            # but we still need to convert to a list to ensure the data is json-serializable
            if is_list(data):
                return data
            return list(data)

        return [
            await _async_transform_recursive(
                d, annotation=annotation, inner_type=inner_type
            )
            for d in data
        ]

    if is_union_type(stripped_type):
        # For union types we run the transformation against all subtypes to ensure that everything is transformed.
        #
        # TODO: there may be edge cases where the same normalized field name will transform to two different names
        # in different subtypes.
        for subtype in get_args(stripped_type):
            data = await _async_transform_recursive(
                data, annotation=annotation, inner_type=subtype
            )
        return data

    if isinstance(data, pydantic.BaseModel):
        return model_dump(data, exclude_unset=True, mode="json")

    annotated_type = _get_annotated_type(annotation)
    if annotated_type is None:
        return data

    # ignore the first argument as it is the actual type
    annotations = get_args(annotated_type)[1:]
    for annotation in annotations:
        if isinstance(annotation, PropertyInfo) and annotation.format is not None:
            return await _async_format_data(
                data, annotation.format, annotation.format_template
            )

    return data


async def _async_format_data(
    data: object, format_: PropertyFormat, format_template: str | None
) -> object:
    if isinstance(data, (date, datetime)):
        if format_ == "iso8601":
            return data.isoformat()

        if format_ == "custom" and format_template is not None:
            return data.strftime(format_template)

    if format_ == "base64" and is_base64_file_input(data):
        binary: str | bytes | None = None

        if isinstance(data, pathlib.Path):
            binary = await anyio.Path(data).read_bytes()
        elif isinstance(data, io.IOBase):
            binary = data.read()

            if isinstance(binary, str):  # type: ignore[unreachable]
                binary = binary.encode()

        if not isinstance(binary, bytes):
            raise RuntimeError(
                f"Could not read bytes from {data}; Received {type(binary)}"
            )

        return base64.b64encode(binary).decode("ascii")

    return data


async def _async_transform_typeddict(
    data: Mapping[str, object],
    expected_type: type,
) -> Mapping[str, object]:
    result: dict[str, object] = {}
    annotations = get_type_hints(expected_type, include_extras=True)
    for key, value in data.items():
        if not is_given(value):
            # we don't need to include `NotGiven` values here as they'll
            # be stripped out before the request is sent anyway
            continue

        type_ = annotations.get(key)
        if type_ is None:
            # we do not have a type annotation for this field, leave it as is
            result[key] = value
        else:
            result[_maybe_transform_key(key, type_)] = await _async_transform_recursive(
                value, annotation=type_
            )
    return result


@lru_cache(maxsize=8096)
def get_type_hints(
    obj: Any,
    globalns: dict[str, Any] | None = None,
    localns: Mapping[str, Any] | None = None,
    include_extras: bool = False,
) -> dict[str, Any]:
    return _get_type_hints(
        obj, globalns=globalns, localns=localns, include_extras=include_extras
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from swarms_client import SwarmsClient, AsyncSwarmsClient
from swarms_client.types import AgentRunResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgent:
    parametrize = pytest.mark.parametrize(
        "client", [False, True], indirect=True, ids=["loose", "strict"]
    )

    @pytest.mark.skip()
    @parametrize
    def test_method_run(self, client: SwarmsClient) -> None:
        agent = client.agent.run()
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_run_with_all_params(self, client: SwarmsClient) -> None:
        agent = client.agent.run(
            agent_config={
                "agent_name": "agent_name",
                "auto_generate_prompt": True,
                "description": "description",
                "max_loops": 0,
                "max_tokens": 0,
                "mcp_url": "mcp_url",
                "model_name": "model_name",
                "role": "role",
                "streaming_on": True,
                "system_prompt": "system_prompt",
                "temperature": 0,
                "tools_list_dictionary": [{"foo": "bar"}],
            },
            history={"foo": "bar"},
            img="img",
            imgs=["string"],
            stream=True,
            task="task",
        )
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_run(self, client: SwarmsClient) -> None:
        response = client.agent.with_raw_response.run()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_run(self, client: SwarmsClient) -> None:
        with client.agent.with_streaming_response.run() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentRunResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAgent:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_run(self, async_client: AsyncSwarmsClient) -> None:
        agent = await async_client.agent.run()
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_run_with_all_params(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        agent = await async_client.agent.run(
            agent_config={
                "agent_name": "agent_name",
                "auto_generate_prompt": True,
                "description": "description",
                "max_loops": 0,
                "max_tokens": 0,
                "mcp_url": "mcp_url",
                "model_name": "model_name",
                "role": "role",
                "streaming_on": True,
                "system_prompt": "system_prompt",
                "temperature": 0,
                "tools_list_dictionary": [{"foo": "bar"}],
            },
            history={"foo": "bar"},
            img="img",
            imgs=["string"],
            stream=True,
            task="task",
        )
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncSwarmsClient) -> None:
        response = await async_client.agent.with_raw_response.run()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentRunResponse, agent, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_run(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        async with async_client.agent.with_streaming_response.run() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentRunResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rate import (
    RateResource,
    AsyncRateResource,
    RateResourceWithRawResponse,
    AsyncRateResourceWithRawResponse,
    RateResourceWithStreamingResponse,
    AsyncRateResourceWithStreamingResponse,
)
from .client import (
    ClientResource,
    AsyncClientResource,
    ClientResourceWithRawResponse,
    AsyncClientResourceWithRawResponse,
    ClientResourceWithStreamingResponse,
    AsyncClientResourceWithStreamingResponse,
)

__all__ = [
    "RateResource",
    "AsyncRateResource",
    "RateResourceWithRawResponse",
    "AsyncRateResourceWithRawResponse",
    "RateResourceWithStreamingResponse",
    "AsyncRateResourceWithStreamingResponse",
    "ClientResource",
    "AsyncClientResource",
    "ClientResourceWithRawResponse",
    "AsyncClientResourceWithRawResponse",
    "ClientResourceWithStreamingResponse",
    "AsyncClientResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)

__all__ = [
    "BatchResource",
    "AsyncBatchResource",
    "BatchResourceWithRawResponse",
    "AsyncBatchResourceWithRawResponse",
    "BatchResourceWithStreamingResponse",
    "AsyncBatchResourceWithStreamingResponse",
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional

import httpx

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from ...types import agent_run_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.agent_spec_param import AgentSpecParam
from ...types.agent_run_response import AgentRunResponse

__all__ = ["AgentResource", "AsyncAgentResource"]


class AgentResource(SyncAPIResource):
    @cached_property
    def batch(self) -> BatchResource:
        return BatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return AgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return AgentResourceWithStreamingResponse(self)

    def run(
        self,
        *,
        agent_config: Optional[AgentSpecParam] | NotGiven = NOT_GIVEN,
        history: (
            Union[Dict[str, object], Iterable[Dict[str, str]], None] | NotGiven
        ) = NOT_GIVEN,
        img: Optional[str] | NotGiven = NOT_GIVEN,
        imgs: Optional[List[str]] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        task: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AgentRunResponse:
        """
        Run an agent with the specified task.

        Args:
          agent_config: The configuration of the agent to be completed.

          history: The history of the agent's previous tasks and responses. Can be either a
              dictionary or a list of message objects.

          img: An optional image URL that may be associated with the agent's task or
              representation.

          imgs: A list of image URLs that may be associated with the agent's task or
              representation.

          stream: A flag indicating whether the agent should stream its output.

          task: The task to be completed by the agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/agent/completions",
            body=maybe_transform(
                {
                    "agent_config": agent_config,
                    "history": history,
                    "img": img,
                    "imgs": imgs,
                    "stream": stream,
                    "task": task,
                },
                agent_run_params.AgentRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=AgentRunResponse,
        )


class AsyncAgentResource(AsyncAPIResource):
    @cached_property
    def batch(self) -> AsyncBatchResource:
        return AsyncBatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return AsyncAgentResourceWithStreamingResponse(self)

    async def run(
        self,
        *,
        agent_config: Optional[AgentSpecParam] | NotGiven = NOT_GIVEN,
        history: (
            Union[Dict[str, object], Iterable[Dict[str, str]], None] | NotGiven
        ) = NOT_GIVEN,
        img: Optional[str] | NotGiven = NOT_GIVEN,
        imgs: Optional[List[str]] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        task: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AgentRunResponse:
        """
        Run an agent with the specified task.

        Args:
          agent_config: The configuration of the agent to be completed.

          history: The history of the agent's previous tasks and responses. Can be either a
              dictionary or a list of message objects.

          img: An optional image URL that may be associated with the agent's task or
              representation.

          imgs: A list of image URLs that may be associated with the agent's task or
              representation.

          stream: A flag indicating whether the agent should stream its output.

          task: The task to be completed by the agent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/agent/completions",
            body=await async_maybe_transform(
                {
                    "agent_config": agent_config,
                    "history": history,
                    "img": img,
                    "imgs": imgs,
                    "stream": stream,
                    "task": task,
                },
                agent_run_params.AgentRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=AgentRunResponse,
        )


class AgentResourceWithRawResponse:
    def __init__(self, agent: AgentResource) -> None:
        self._agent = agent

        self.run = to_raw_response_wrapper(
            agent.run,
        )

    @cached_property
    def batch(self) -> BatchResourceWithRawResponse:
        return BatchResourceWithRawResponse(self._agent.batch)


class AsyncAgentResourceWithRawResponse:
    def __init__(self, agent: AsyncAgentResource) -> None:
        self._agent = agent

        self.run = async_to_raw_response_wrapper(
            agent.run,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithRawResponse:
        return AsyncBatchResourceWithRawResponse(self._agent.batch)


class AgentResourceWithStreamingResponse:
    def __init__(self, agent: AgentResource) -> None:
        self._agent = agent

        self.run = to_streamed_response_wrapper(
            agent.run,
        )

    @cached_property
    def batch(self) -> BatchResourceWithStreamingResponse:
        return BatchResourceWithStreamingResponse(self._agent.batch)


class AsyncAgentResourceWithStreamingResponse:
    def __init__(self, agent: AsyncAgentResource) -> None:
        self._agent = agent

        self.run = async_to_streamed_response_wrapper(
            agent.run,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithStreamingResponse:
        return AsyncBatchResourceWithStreamingResponse(self._agent.batch)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["AgentSpecParam"]


class AgentSpecParam(TypedDict, total=False):
    agent_name: Required[Optional[str]]
    """
    The unique name assigned to the agent, which identifies its role and
    functionality within the swarm.
    """

    auto_generate_prompt: Optional[bool]
    """
    A flag indicating whether the agent should automatically create prompts based on
    the task requirements.
    """

    description: Optional[str]
    """
    A detailed explanation of the agent's purpose, capabilities, and any specific
    tasks it is designed to perform.
    """

    max_loops: Optional[int]
    """
    The maximum number of times the agent is allowed to repeat its task, enabling
    iterative processing if necessary.
    """

    max_tokens: Optional[int]
    """
    The maximum number of tokens that the agent is allowed to generate in its
    responses, limiting output length.
    """

    mcp_url: Optional[str]
    """The URL of the MCP server that the agent can use to complete its task."""

    model_name: Optional[str]
    """
    The name of the AI model that the agent will utilize for processing tasks and
    generating outputs. For example: gpt-4o, gpt-4o-mini, openai/o3-mini
    """

    role: Optional[str]
    """
    The designated role of the agent within the swarm, which influences its behavior
    and interaction with other agents.
    """

    streaming_on: Optional[bool]
    """A flag indicating whether the agent should stream its output."""

    system_prompt: Optional[str]
    """
    The initial instruction or context provided to the agent, guiding its behavior
    and responses during execution.
    """

    temperature: Optional[float]
    """
    A parameter that controls the randomness of the agent's output; lower values
    result in more deterministic responses.
    """

    tools_list_dictionary: Optional[Iterable[Dict[str, object]]]
    """A dictionary of tools that the agent can use to complete its task."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rate import (
    RateResource,
    AsyncRateResource,
    RateResourceWithRawResponse,
    AsyncRateResourceWithRawResponse,
    RateResourceWithStreamingResponse,
    AsyncRateResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ClientResource", "AsyncClientResource"]


class ClientResource(SyncAPIResource):
    @cached_property
    def rate(self) -> RateResource:
        return RateResource(self._client)

    @cached_property
    def with_raw_response(self) -> ClientResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return ClientResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClientResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return ClientResourceWithStreamingResponse(self)


class AsyncClientResource(AsyncAPIResource):
    @cached_property
    def rate(self) -> AsyncRateResource:
        return AsyncRateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncClientResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncClientResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClientResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return AsyncClientResourceWithStreamingResponse(self)


class ClientResourceWithRawResponse:
    def __init__(self, client: ClientResource) -> None:
        self._client = client

    @cached_property
    def rate(self) -> RateResourceWithRawResponse:
        return RateResourceWithRawResponse(self._client.rate)


class AsyncClientResourceWithRawResponse:
    def __init__(self, client: AsyncClientResource) -> None:
        self._client = client

    @cached_property
    def rate(self) -> AsyncRateResourceWithRawResponse:
        return AsyncRateResourceWithRawResponse(self._client.rate)


class ClientResourceWithStreamingResponse:
    def __init__(self, client: ClientResource) -> None:
        self._client = client

    @cached_property
    def rate(self) -> RateResourceWithStreamingResponse:
        return RateResourceWithStreamingResponse(self._client.rate)


class AsyncClientResourceWithStreamingResponse:
    def __init__(self, client: AsyncClientResource) -> None:
        self._client = client

    @cached_property
    def rate(self) -> AsyncRateResourceWithStreamingResponse:
        return AsyncRateResourceWithStreamingResponse(self._client.rate)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from swarms_client import SwarmsClient, AsyncSwarmsClient
from swarms_client.types.agent import BatchRunResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBatch:
    parametrize = pytest.mark.parametrize(
        "client", [False, True], indirect=True, ids=["loose", "strict"]
    )

    @pytest.mark.skip()
    @parametrize
    def test_method_run(self, client: SwarmsClient) -> None:
        batch = client.agent.batch.run(
            body=[{}],
        )
        assert_matches_type(BatchRunResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_run(self, client: SwarmsClient) -> None:
        response = client.agent.batch.with_raw_response.run(
            body=[{}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = response.parse()
        assert_matches_type(BatchRunResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_run(self, client: SwarmsClient) -> None:
        with client.agent.batch.with_streaming_response.run(
            body=[{}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = response.parse()
            assert_matches_type(BatchRunResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBatch:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_run(self, async_client: AsyncSwarmsClient) -> None:
        batch = await async_client.agent.batch.run(
            body=[{}],
        )
        assert_matches_type(BatchRunResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncSwarmsClient) -> None:
        response = await async_client.agent.batch.with_raw_response.run(
            body=[{}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = await response.parse()
        assert_matches_type(BatchRunResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_run(
        self, async_client: AsyncSwarmsClient
    ) -> None:
        async with async_client.agent.batch.with_streaming_response.run(
            body=[{}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = await response.parse()
            assert_matches_type(BatchRunResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal

import httpx

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from ...types import swarm_run_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.agent_spec_param import AgentSpecParam
from ...types.swarm_run_response import SwarmRunResponse
from ...types.swarm_get_logs_response import SwarmGetLogsResponse
from ...types.swarm_check_available_response import SwarmCheckAvailableResponse

__all__ = ["SwarmsResource", "AsyncSwarmsResource"]


class SwarmsResource(SyncAPIResource):
    @cached_property
    def batch(self) -> BatchResource:
        return BatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> SwarmsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return SwarmsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SwarmsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return SwarmsResourceWithStreamingResponse(self)

    def check_available(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmCheckAvailableResponse:
        """Check the available swarm types."""
        return self._get(
            "/v1/swarms/available",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmCheckAvailableResponse,
        )

    def get_logs(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmGetLogsResponse:
        """
        Get all API request logs for the user associated with the provided API key,
        excluding any logs that contain a client_ip field in their data.
        """
        return self._get(
            "/v1/swarm/logs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmGetLogsResponse,
        )

    def run(
        self,
        *,
        agents: Optional[Iterable[AgentSpecParam]] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        img: Optional[str] | NotGiven = NOT_GIVEN,
        max_loops: Optional[int] | NotGiven = NOT_GIVEN,
        messages: (
            Union[Iterable[Dict[str, object]], Dict[str, object], None] | NotGiven
        ) = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        rearrange_flow: Optional[str] | NotGiven = NOT_GIVEN,
        return_history: Optional[bool] | NotGiven = NOT_GIVEN,
        rules: Optional[str] | NotGiven = NOT_GIVEN,
        service_tier: Optional[str] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        swarm_type: (
            Optional[
                Literal[
                    "AgentRearrange",
                    "MixtureOfAgents",
                    "SpreadSheetSwarm",
                    "SequentialWorkflow",
                    "ConcurrentWorkflow",
                    "GroupChat",
                    "MultiAgentRouter",
                    "AutoSwarmBuilder",
                    "HiearchicalSwarm",
                    "auto",
                    "MajorityVoting",
                    "MALT",
                    "DeepResearchSwarm",
                    "CouncilAsAJudge",
                    "InteractiveGroupChat",
                ]
            ]
            | NotGiven
        ) = NOT_GIVEN,
        task: Optional[str] | NotGiven = NOT_GIVEN,
        tasks: Optional[List[str]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmRunResponse:
        """
        Run a swarm with the specified task.

        Args:
          agents: A list of agents or specifications that define the agents participating in the
              swarm.

          description: A comprehensive description of the swarm's objectives, capabilities, and
              intended outcomes.

          img: An optional image URL that may be associated with the swarm's task or
              representation.

          max_loops: The maximum number of execution loops allowed for the swarm, enabling repeated
              processing if needed.

          messages: A list of messages that the swarm should complete.

          name: The name of the swarm, which serves as an identifier for the group of agents and
              their collective task.

          rearrange_flow: Instructions on how to rearrange the flow of tasks among agents, if applicable.

          return_history: A flag indicating whether the swarm should return its execution history along
              with the final output.

          rules: Guidelines or constraints that govern the behavior and interactions of the
              agents within the swarm.

          service_tier: The service tier to use for processing. Options: 'standard' (default) or 'flex'
              for lower cost but slower processing.

          stream: A flag indicating whether the swarm should stream its output.

          swarm_type: The classification of the swarm, indicating its operational style and
              methodology.

          task: The specific task or objective that the swarm is designed to accomplish.

          tasks: A list of tasks that the swarm should complete.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/swarm/completions",
            body=maybe_transform(
                {
                    "agents": agents,
                    "description": description,
                    "img": img,
                    "max_loops": max_loops,
                    "messages": messages,
                    "name": name,
                    "rearrange_flow": rearrange_flow,
                    "return_history": return_history,
                    "rules": rules,
                    "service_tier": service_tier,
                    "stream": stream,
                    "swarm_type": swarm_type,
                    "task": task,
                    "tasks": tasks,
                },
                swarm_run_params.SwarmRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmRunResponse,
        )


class AsyncSwarmsResource(AsyncAPIResource):
    @cached_property
    def batch(self) -> AsyncBatchResource:
        return AsyncBatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSwarmsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSwarmsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSwarmsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/The-Swarm-Corporation/swarms-sdk#with_streaming_response
        """
        return AsyncSwarmsResourceWithStreamingResponse(self)

    async def check_available(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmCheckAvailableResponse:
        """Check the available swarm types."""
        return await self._get(
            "/v1/swarms/available",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmCheckAvailableResponse,
        )

    async def get_logs(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmGetLogsResponse:
        """
        Get all API request logs for the user associated with the provided API key,
        excluding any logs that contain a client_ip field in their data.
        """
        return await self._get(
            "/v1/swarm/logs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmGetLogsResponse,
        )

    async def run(
        self,
        *,
        agents: Optional[Iterable[AgentSpecParam]] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        img: Optional[str] | NotGiven = NOT_GIVEN,
        max_loops: Optional[int] | NotGiven = NOT_GIVEN,
        messages: (
            Union[Iterable[Dict[str, object]], Dict[str, object], None] | NotGiven
        ) = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        rearrange_flow: Optional[str] | NotGiven = NOT_GIVEN,
        return_history: Optional[bool] | NotGiven = NOT_GIVEN,
        rules: Optional[str] | NotGiven = NOT_GIVEN,
        service_tier: Optional[str] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        swarm_type: (
            Optional[
                Literal[
                    "AgentRearrange",
                    "MixtureOfAgents",
                    "SpreadSheetSwarm",
                    "SequentialWorkflow",
                    "ConcurrentWorkflow",
                    "GroupChat",
                    "MultiAgentRouter",
                    "AutoSwarmBuilder",
                    "HiearchicalSwarm",
                    "auto",
                    "MajorityVoting",
                    "MALT",
                    "DeepResearchSwarm",
                    "CouncilAsAJudge",
                    "InteractiveGroupChat",
                ]
            ]
            | NotGiven
        ) = NOT_GIVEN,
        task: Optional[str] | NotGiven = NOT_GIVEN,
        tasks: Optional[List[str]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SwarmRunResponse:
        """
        Run a swarm with the specified task.

        Args:
          agents: A list of agents or specifications that define the agents participating in the
              swarm.

          description: A comprehensive description of the swarm's objectives, capabilities, and
              intended outcomes.

          img: An optional image URL that may be associated with the swarm's task or
              representation.

          max_loops: The maximum number of execution loops allowed for the swarm, enabling repeated
              processing if needed.

          messages: A list of messages that the swarm should complete.

          name: The name of the swarm, which serves as an identifier for the group of agents and
              their collective task.

          rearrange_flow: Instructions on how to rearrange the flow of tasks among agents, if applicable.

          return_history: A flag indicating whether the swarm should return its execution history along
              with the final output.

          rules: Guidelines or constraints that govern the behavior and interactions of the
              agents within the swarm.

          service_tier: The service tier to use for processing. Options: 'standard' (default) or 'flex'
              for lower cost but slower processing.

          stream: A flag indicating whether the swarm should stream its output.

          swarm_type: The classification of the swarm, indicating its operational style and
              methodology.

          task: The specific task or objective that the swarm is designed to accomplish.

          tasks: A list of tasks that the swarm should complete.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/swarm/completions",
            body=await async_maybe_transform(
                {
                    "agents": agents,
                    "description": description,
                    "img": img,
                    "max_loops": max_loops,
                    "messages": messages,
                    "name": name,
                    "rearrange_flow": rearrange_flow,
                    "return_history": return_history,
                    "rules": rules,
                    "service_tier": service_tier,
                    "stream": stream,
                    "swarm_type": swarm_type,
                    "task": task,
                    "tasks": tasks,
                },
                swarm_run_params.SwarmRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=SwarmRunResponse,
        )


class SwarmsResourceWithRawResponse:
    def __init__(self, swarms: SwarmsResource) -> None:
        self._swarms = swarms

        self.check_available = to_raw_response_wrapper(
            swarms.check_available,
        )
        self.get_logs = to_raw_response_wrapper(
            swarms.get_logs,
        )
        self.run = to_raw_response_wrapper(
            swarms.run,
        )

    @cached_property
    def batch(self) -> BatchResourceWithRawResponse:
        return BatchResourceWithRawResponse(self._swarms.batch)


class AsyncSwarmsResourceWithRawResponse:
    def __init__(self, swarms: AsyncSwarmsResource) -> None:
        self._swarms = swarms

        self.check_available = async_to_raw_response_wrapper(
            swarms.check_available,
        )
        self.get_logs = async_to_raw_response_wrapper(
            swarms.get_logs,
        )
        self.run = async_to_raw_response_wrapper(
            swarms.run,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithRawResponse:
        return AsyncBatchResourceWithRawResponse(self._swarms.batch)


class SwarmsResourceWithStreamingResponse:
    def __init__(self, swarms: SwarmsResource) -> None:
        self._swarms = swarms

        self.check_available = to_streamed_response_wrapper(
            swarms.check_available,
        )
        self.get_logs = to_streamed_response_wrapper(
            swarms.get_logs,
        )
        self.run = to_streamed_response_wrapper(
            swarms.run,
        )

    @cached_property
    def batch(self) -> BatchResourceWithStreamingResponse:
        return BatchResourceWithStreamingResponse(self._swarms.batch)


class AsyncSwarmsResourceWithStreamingResponse:
    def __init__(self, swarms: AsyncSwarmsResource) -> None:
        self._swarms = swarms

        self.check_available = async_to_streamed_response_wrapper(
            swarms.check_available,
        )
        self.get_logs = async_to_streamed_response_wrapper(
            swarms.get_logs,
        )
        self.run = async_to_streamed_response_wrapper(
            swarms.run,
        )

    @cached_property
    def batch(self) -> AsyncBatchResourceWithStreamingResponse:
        return AsyncBatchResourceWithStreamingResponse(self._swarms.batch)
